@@ -38,6 +38,7 @@
 ****************************************************************************/
 
 #include "qquicktaphandler_p.h"
+#include "qquicksinglepointhandler_p_p.h"
 #include <qpa/qplatformtheme.h>
 #include <private/qguiapplication_p.h>
 #include <QtGui/qstylehints.h>
@@ -77,8 +78,8 @@ int QQuickTapHandler::m_touchMultiTapDistanceSquared(-1);
     \l gesturePolicy to \c TapHandler.ReleaseWithinBounds.
 
     For multi-tap gestures (double-tap, triple-tap etc.), the distance moved
-    must not exceed QPlatformTheme::MouseDoubleClickDistance with mouse and
-    QPlatformTheme::TouchDoubleTapDistance with touch, and the time between
+    must not exceed QStyleHints::mouseDoubleClickDistance() with mouse and
+    QStyleHints::touchDoubleTapDistance() with touch, and the time between
     taps must not exceed QStyleHints::mouseDoubleClickInterval().
 
     \sa MouseArea
@@ -89,11 +90,9 @@ QQuickTapHandler::QQuickTapHandler(QQuickItem *parent)
 {
     if (m_mouseMultiClickDistanceSquared < 0) {
         m_multiTapInterval = qApp->styleHints()->mouseDoubleClickInterval() / 1000.0;
-        m_mouseMultiClickDistanceSquared = QGuiApplicationPrivate::platformTheme()->
-                    themeHint(QPlatformTheme::MouseDoubleClickDistance).toInt();
+        m_mouseMultiClickDistanceSquared = qApp->styleHints()->mouseDoubleClickDistance();
         m_mouseMultiClickDistanceSquared *= m_mouseMultiClickDistanceSquared;
-        m_touchMultiTapDistanceSquared = QGuiApplicationPrivate::platformTheme()->
-                    themeHint(QPlatformTheme::TouchDoubleTapDistance).toInt();
+        m_touchMultiTapDistanceSquared = qApp->styleHints()->touchDoubleTapDistance();
         m_touchMultiTapDistanceSquared *= m_touchMultiTapDistanceSquared;
     }
 }
@@ -316,7 +315,7 @@ void QQuickTapHandler::setPressed(bool press, bool cancel, QQuickEventPoint *poi
             setExclusiveGrab(point, false);
             // In case there is a filtering parent (Flickable), we should not give up the passive grab,
             // so that it can continue to filter future events.
-            reset();
+            d_func()->reset();
             emit pointChanged();
         }
     }
@@ -386,7 +385,7 @@ void QQuickTapHandler::updateTimeHeld()
     That is, if you press and release a touchpoint or button within a time
     period less than \l longPressThreshold, while any movement does not exceed
     the drag threshold, then the \c tapped signal will be emitted at the time
-    of release.  The \c eventPoint signal parameter contains information
+    of release.  The \a eventPoint signal parameter contains information
     from the release event about the point that was tapped:
 
     \snippet pointerHandlers/tapHandlerOnTapped.qml 0
@@ -399,7 +398,7 @@ void QQuickTapHandler::updateTimeHeld()
     This signal is emitted when the \c parent Item is tapped once.
     After an amount of time greater than QStyleHints::mouseDoubleClickInterval,
     it can be tapped again; but if the time until the next tap is less,
-    \l tapCount will increase. The \c eventPoint signal parameter contains
+    \l tapCount will increase. The \a eventPoint signal parameter contains
     information from the release event about the point that was tapped.
 */
 
@@ -408,16 +407,16 @@ void QQuickTapHandler::updateTimeHeld()
     \since 5.11
 
     This signal is emitted when the \c parent Item is tapped twice within a
-    short span of time (QStyleHints::mouseDoubleClickInterval) and distance
-    (QPlatformTheme::MouseDoubleClickDistance or
-    QPlatformTheme::TouchDoubleTapDistance). This signal always occurs after
-    \l singleTapped, \l tapped, and \l tapCountChanged. The \c eventPoint
+    short span of time (QStyleHints::mouseDoubleClickInterval()) and distance
+    (QStyleHints::mouseDoubleClickDistance() or
+    QStyleHints::touchDoubleTapDistance()). This signal always occurs after
+    \l singleTapped, \l tapped, and \l tapCountChanged. The \a eventPoint
     signal parameter contains information from the release event about the
     point that was tapped.
 */
 
 /*!
-    \qmlsignal QtQuick::TapHandler::longPressed
+    \qmlsignal QtQuick::TapHandler::longPressed()
 
     This signal is emitted when the \c parent Item is pressed and held for a
     time period greater than \l longPressThreshold. That is, if you press and
@@ -427,7 +426,7 @@ void QQuickTapHandler::updateTimeHeld()
 */
 
 /*!
-    \qmlsignal QtQuick::TapHandler::tapCountChanged
+    \qmlsignal QtQuick::TapHandler::tapCountChanged()
 
     This signal is emitted when the \c parent Item is tapped once or more (within
     a specified time and distance span) and when the present \c tapCount differs
